@@ -16,22 +16,22 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* This file must not be used in other header files. */
+#ifndef _NET6_COMMON_HPP_
+#define _NET6_COMMON_HPP_
 
-#include "config.hpp"
+#include "gettext_package.hpp"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
+namespace net6
+{
+
+/** Call to initialise gettext and to be able to get translated messages by _()
+ */
+void init_gettext(gettext_package& package);
+
+/** Translates the given message in the net6 message catalog.
+ */
+const char* _(const char* msgid);
 
-template<typename char_type>
-inline const char_type* _(const char_type* text) {
-	return dgettext(PACKAGE, text);
 }
-#else
-/* This functions is a no-op for systems without a useable GNU gettext. */
-template<typename char_type>
-inline const char_type* _(const char_type* text) {
-	return text;
-}
-#endif
 
+#endif // _NET6_COMMON_HPP_
