@@ -20,12 +20,15 @@
 #define _NET6_MAIN_HPP_
 
 #include "non_copyable.hpp"
+#include "gettext_package.hpp"
 
+/** The main net6 namespace where all net6 stuff belongs to.
+ */
 namespace net6
 {
 
 /** net6 main object. Every net6 capable program needs one of these objects
- * to (de)initialize the library
+ * to (de)initialise the library.
  */
 	
 class main : private non_copyable
@@ -36,6 +39,7 @@ public:
 
 private:
 	static unsigned int refcount;
+	static gettext_package* package;
 };
 
 }
