@@ -16,15 +16,29 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "config.hpp"
+
 #include "error.hpp"
 #include "socket.hpp"
 
+#ifdef WIN32
+# define WIN32_CAST_FIX(a) (static_cast<char*>(a) )
+# define WIN32_CCAST_FIX(a) (static_cast<const char*>(a) )
+#else
+# define WIN32_CAST_FIX(a) (a)
+# define WIN32_CCAST_FIX(a) (a)
+# include <unistd.h>
+#endif
+
 namespace
 {
 	int address_to_protocol(int af)
 	{
 		switch(af)
 		{
+#ifndef WIN32
+		case AF_UNIX: return PF_UNIX;
+#endif
 		case AF_INET: return PF_INET;
 		case AF_INET6: return PF_INET6;
 		default:
@@ -33,158 +47,123 @@ namespace
 			);
 		}
 	}
-}
 
-#ifdef WIN32
-#define WIN32_CAST_FIX(a) (static_cast<char*>(a) )
-#define WIN32_CCAST_FIX(a) (static_cast<const char*>(a) )
-#else
-#define WIN32_CAST_FIX(a) (a)
-#define WIN32_CCAST_FIX(a) (a)
+	void set_nosigpipe(net6::tcp_socket::socket_type socket)
+	{
+#if HAVE_SO_NOSIGPIPE
+		// Mac OS X
+		int value = 1;
+		if(setsockopt(socket, SOL_SOCKET, SO_NOSIGPIPE, &value,
+				sizeof(int)) == -1)
+			throw net6::error(net6::error::SYSTEM);
 #endif
+	}
 
 #ifndef WIN32
-namespace { const int INVALID_SOCKET = -1; }
+	const int INVALID_SOCKET = -1;
 #endif
+}
 
-net6::socket::socket(int domain, int type, int protocol)
- : data(new socket_data)
+net6::socket::socket(int domain, int type, int protocol):
+	sock(::socket(domain, type, protocol) )
 {
-	data->sock = ::socket(domain, type, protocol);
-	data->refcount = 1;
-
-	if(data->sock == INVALID_SOCKET)
+	if(sock == INVALID_SOCKET)
 		throw error(error::SYSTEM);
 }
 
-net6::socket::socket(socket_type c_object)
- : data(new socket_data)
+net6::socket::socket(socket_type c_object):
+	sock(c_object)
 {
-	data->sock = c_object;
-	data->refcount = 1;
-}
-
-net6::socket::socket(const socket& other)
- : data(other.data)
-{
-	data->refcount ++;
 }
 
 net6::socket::~socket()
 {
-	data->refcount --;
-
-	if(!data->refcount)
+	if(sock != INVALID_SOCKET)
 	{
 #ifdef WIN32
 		closesocket(cobj() );
 #else
 		close(cobj() );
 #endif
-		delete data;
 	}
 }
 
-net6::socket& net6::socket::operator=(const socket& other)
+void net6::socket::invalidate()
 {
-	if(this == &other)
-		return *this;
-
-	data->refcount --;
-	if(!data->refcount)
-	{
-#ifdef WIN32
-		closesocket(cobj() );
-#else
-		close(cobj() );
-#endif
-		delete data;
-	}
-
-	data = other.data;
-	data->refcount ++;
-	
-	return *this;
+	sock = INVALID_SOCKET;
 }
 
-void net6::socket::on_io(condition cond)
-{
-	data->signal_io.emit(cond);
-}
-
-net6::tcp_socket::tcp_socket(const address& addr)
- : socket(address_to_protocol(addr.get_family()), SOCK_STREAM, IPPROTO_TCP)
+net6::tcp_socket::tcp_socket(const address& addr):
+	socket(address_to_protocol(addr.get_family()), SOCK_STREAM, 0)
 {
 }
 
-net6::tcp_socket::tcp_socket(socket_type c_object)
- : socket(c_object)
+net6::tcp_socket::tcp_socket(socket_type c_object):
+	socket(c_object)
 {
 }
 
-net6::tcp_socket::tcp_socket(const tcp_socket& other)
- : socket(other)
+net6::tcp_client_socket::tcp_client_socket(const address& addr):
+	tcp_socket(addr)
 {
-}
-
-net6::tcp_socket::~tcp_socket()
-{
-}
-
-net6::tcp_socket& net6::tcp_socket::operator=(const tcp_socket& other)
-{
-	return static_cast<tcp_socket&>(socket::operator=(other) );
-}
-
-net6::tcp_client_socket::tcp_client_socket(const address& addr)
- : tcp_socket(addr)
-{
-	if(connect(cobj(), addr.cobj(), addr.get_size()) == -1)
+	if(::connect(cobj(), addr.cobj(), addr.get_size()) == -1)
 		throw error(net6::error::SYSTEM);
+
+	set_nosigpipe(cobj() );
 }
 
-net6::tcp_client_socket::tcp_client_socket(socket_type c_object)
- : tcp_socket(c_object)
-{
-}
-
-net6::tcp_client_socket::tcp_client_socket(const tcp_client_socket& other)
- : tcp_socket(other)
+net6::tcp_client_socket::tcp_client_socket(socket_type c_object):
+	tcp_socket(c_object)
 {
+	set_nosigpipe(c_object);
 }
 
 net6::tcp_client_socket::~tcp_client_socket()
 {
 }
 
-net6::tcp_client_socket&
-net6::tcp_client_socket::operator=(const tcp_client_socket& other)
+net6::socket::size_type net6::tcp_client_socket::send(const void* buf,
+                                                      size_type len) const
 {
-	return static_cast<tcp_client_socket&>(tcp_socket::operator=(other) );
-}
+	ssize_t result = ::send(
+		cobj(),
+		WIN32_CCAST_FIX(buf),
+		len,
+#ifdef HAVE_MSG_NOSIGNAL
+		MSG_NOSIGNAL
+#else
+		0
+#endif
+	);
 
-net6::socket::size_type
-net6::tcp_client_socket::send(const void* buf, size_type len) const
-{
-	ssize_t result = ::send(cobj(), WIN32_CCAST_FIX(buf), len, 0);
 	if(result < 0)
 		throw error(net6::error::SYSTEM);
 
 	return result;
 }
 
-net6::socket::size_type
-net6::tcp_client_socket::recv(void* buf, size_type len) const
+net6::socket::size_type net6::tcp_client_socket::recv(void* buf,
+                                                      size_type len) const
 {
-	ssize_t result = ::recv(cobj(), WIN32_CAST_FIX(buf), len, 0);
+	ssize_t result = ::recv(
+		cobj(),
+		WIN32_CAST_FIX(buf),
+		len,
+#ifdef HAVE_MSG_NOSIGNAL
+		MSG_NOSIGNAL
+#else
+		0
+#endif
+	);
+
 	if(result < 0)
 		throw error(net6::error::SYSTEM);
 
 	return result;
 }
 
-net6::tcp_server_socket::tcp_server_socket(const address& bind_addr)
- : tcp_socket(bind_addr)
+net6::tcp_server_socket::tcp_server_socket(const address& bind_addr):
+	tcp_socket(bind_addr)
 {
 	if(bind(cobj(), bind_addr.cobj(), bind_addr.get_size()) == -1)
 		throw error(net6::error::SYSTEM);
@@ -192,66 +171,46 @@ net6::tcp_server_socket::tcp_server_socket(const address& bind_addr)
 		throw error(net6::error::SYSTEM);
 }
 
-net6::tcp_server_socket::tcp_server_socket(socket_type c_object)
- : tcp_socket(c_object)
+net6::tcp_server_socket::tcp_server_socket(socket_type c_object):
+	tcp_socket(c_object)
 {
 }
 
-net6::tcp_server_socket::tcp_server_socket(const tcp_server_socket& other)
- : tcp_socket(other)
-{
-}
-
-net6::tcp_server_socket::~tcp_server_socket()
-{
-}
-
-net6::tcp_server_socket&
-net6::tcp_server_socket::operator=(const tcp_server_socket& other)
-{
-	return static_cast<tcp_server_socket&>(tcp_socket::operator=(other) );
-}
-
-net6::tcp_client_socket net6::tcp_server_socket::accept() const
+std::auto_ptr<net6::tcp_client_socket> net6::tcp_server_socket::accept() const
 {
 	socket_type new_sock = ::accept(cobj(), NULL, NULL);
 	if(new_sock == INVALID_SOCKET)
 		throw error(net6::error::SYSTEM);
 
-	return tcp_client_socket(new_sock);
+	return std::auto_ptr<tcp_client_socket>(
+		new tcp_client_socket(new_sock)
+	);
 }
 
-net6::tcp_client_socket net6::tcp_server_socket::accept(address& from) const
+std::auto_ptr<net6::tcp_client_socket>
+net6::tcp_server_socket::accept(address& from) const
 {
 	socklen_t sock_size = from.get_size();
 	socket_type new_sock = ::accept(cobj(), from.cobj(), &sock_size);
 	if(new_sock == INVALID_SOCKET)
 		throw error(net6::error::SYSTEM);
 
-	return tcp_client_socket(new_sock);
+	return std::auto_ptr<tcp_client_socket>(
+		new tcp_client_socket(new_sock)
+	);
 }
 
-net6::udp_socket::udp_socket(const address& bind_addr)
- : socket(address_to_protocol(bind_addr.get_family()), SOCK_DGRAM, IPPROTO_UDP)
+net6::udp_socket::udp_socket(const address& bind_addr):
+	socket(
+		address_to_protocol(bind_addr.get_family()),
+		SOCK_DGRAM,
+		IPPROTO_UDP
+	)
 {
-	if(bind(cobj(), bind_addr.cobj(), bind_addr.get_size()) == -1)
+	if(::bind(cobj(), bind_addr.cobj(), bind_addr.get_size()) == -1)
 		throw error(net6::error::SYSTEM);
 }
 
-net6::udp_socket::udp_socket(const udp_socket& other)
- : socket(other)
-{
-}
-
-net6::udp_socket::~udp_socket()
-{
-}
-
-net6::udp_socket& net6::udp_socket::operator=(const udp_socket& other)
-{
-	return static_cast<udp_socket&>(udp_socket::operator=(other) );
-}
-
 void net6::udp_socket::set_target(const address& addr)
 {
 	if(connect(cobj(), addr.cobj(), addr.get_size()) == -1)
@@ -306,4 +265,3 @@ net6::udp_socket::recv(void* buf, size_type len, address& from) const
 
 	return result;
 }
-
